package moje_jdbc;

import java.sql.*;
import java.util.*;

public class Data {
  private static Random rRandom = new Random(100);

  static int random(int i) {
    i = rRandom.nextInt() % i;
    return i < 0 ? -i : i;
  }

  static String random(String s[]) {
    return s[random(s.length)];
  }

  static void createTestTables(Statement sStatement) {
    String demo[] = {
            "DROP TABLE Item",
	    "DROP TABLE Invoice",
            "DROP TABLE Product",
	    "DROP TABLE Customer",
            "CREATE TABLE Customer(ID INTEGER PRIMARY KEY,FirstName VARCHAR2(20),"
            + "LastName VARCHAR2(20),Street VARCHAR2(20),City VARCHAR2(20))",
            "CREATE TABLE Product(ID INTEGER PRIMARY KEY,Name VARCHAR2(20),"
            + "Price NUMBER(10,2))",
            "CREATE TABLE Invoice(ID INTEGER PRIMARY KEY,CustomerID INTEGER,"
            + "Total NUMBER(10,2))",
            "CREATE TABLE Item(InvoiceID INTEGER,Item INTEGER,"
            + "ProductID INTEGER,Quantity INTEGER,Cost NUMBER(10,2),"
            + "PRIMARY KEY(InvoiceID,Item))"
        };

    for (int i = 0; i < demo.length; i++) {

      // drop table may fail
      try {
        sStatement.execute(demo[i]);
      } catch (SQLException e) {
	System.out.println(e);
      }
    }
  }

  static void createConstraints(Statement sStatement) throws SQLException {
    String demo[] = {
	    "ALTER TABLE Invoice ADD CONSTRAINT Invoice_Customer_FK"
	    + " FOREIGN KEY (CustomerId) REFERENCES Customer(ID) ON DELETE CASCADE",
	    "ALTER TABLE Item ADD CONSTRAINT Item_Invoice_FK"
            + " FOREIGN KEY (InvoiceId) REFERENCES Invoice(ID) ON DELETE CASCADE",
	    "ALTER TABLE Item ADD CONSTRAINT Item_Product_FK"
	    + " FOREIGN KEY (ProductId) REFERENCES Product(ID) ON DELETE CASCADE"
	};
    for (int i = 0; i < demo.length; i++)
        sStatement.execute(demo[i]);
  }

  static void createTestData(Statement sStatement) throws SQLException {
    String name[] = {
        "White", "Karsen", "Smith", "Ringer", "May", "King", "Fuller",
        "Miller", "Ott", "Sommer", "Schneider", "Steel", "Peterson",
        "Heiniger", "Clancy"
    };
    String firstname[] = {
        "Mary", "James", "Anne", "George", "Sylvia", "Robert", "Janet",
        "Michael", "Andrew", "Bill", "Susanne", "Laura", "Bob", "Julia",
        "John"
    };
    String street[] = {
        "Upland Pl.", "College Av.", "- 20th Ave.", "Seventh Av."
    };
    String city[]   = {
        "New York", "Dallas", "Boston", "Chicago", "Seattle",
        "San Francisco", "Berne", "Oslo", "Paris", "Lyon", "Palo Alto",
        "Olten"
    };
    String product[] = {
        "Iron", "Ice Tea", "Clock", "Chair", "Telephone", "Shoe"
    };
    int    max       = 50;

    for (int i = 0; i < max; i++) {
      sStatement.execute("INSERT INTO Customer VALUES(" + i + ",'"
                         + random(firstname) + "','" + random(name)
                         + "','" + random(554) + " " + random(street)
                         + "','" + random(city) + "')");
      sStatement.execute("INSERT INTO Product VALUES(" + i + ",'"
                         + random(product) + " " + random(product)
                         + "'," + (20 + 2 * random(120)) + ")");
      sStatement.execute("INSERT INTO Invoice VALUES(" + i + ","
                         + random(max) + ",0.0)");
      for (int j = random(20) + 2; j >= 0; j--) {
        sStatement.execute("INSERT INTO Item VALUES(" + i + "," + j
                           + "," + random(max) + ","
                           + (1 + random(24)) + ",1.5)");
      }
    }

    createConstraints(sStatement);
    sStatement.execute("UPDATE Product SET Price=ROUND(Price*.1,2)");
    sStatement.execute("UPDATE Item SET Cost=Cost*"
                       + "(SELECT Price FROM Product prod WHERE ProductID=prod.ID)");
    sStatement.execute("UPDATE Invoice SET Total=(SELECT SUM(Cost*"
                       + "Quantity) FROM Item WHERE InvoiceID=Invoice.ID)");
  }

  public static void main(String args[]) {
    Connection con = null;

    try {
      Class.forName(Common.CLASSNAME).newInstance();
      System.out.println("driver zaladowany");
    }
    catch( Exception e ) {
      System.out.println("Failed to load driver.");
      return;
    }
    try {
      con = DriverManager.getConnection(Common.URL, Common.USERNAME, Common.PASSWORD);
      System.out.println("mamy connection");

      Statement stm = con.createStatement();
      System.out.println("mamy statement");

      createTestTables(stm);
      System.out.println("tabele utworzone");

      createTestData(stm);
      System.out.println("tabele wypelnione");
    }
    catch( Exception e ) {
      e.printStackTrace();
    }
    finally {
      if( con != null ) {
        try { con.close(); }
        catch( Exception e ) { e.printStackTrace(); }
      }
    }
  }
}
